@@ -376,10 +376,10 @@ struct SHELL_Cmd {
  * as well if the line set a a variable */
 class AutoexecObject{
 private:
-	bool installed;
+	bool installed = false;
 	std::string buf;
 public:
-	AutoexecObject():installed(false){ };
+	AutoexecObject() {};
 	void Install(std::string const &in);
 	void InstallBefore(std::string const &in);
 	void Uninstall();
@@ -388,4 +388,6 @@ private:
 	void CreateAutoexec(void);
 };
 
+size_t GetPauseCount();
+
 #endif
