@@ -31,7 +31,9 @@
  * MT safe
  */
 
-#if !C_FLUIDSYNTH && defined(WIN32)
+#define FLUIDINC
+#include "config.h"
+#if !C_FLUIDSYNTH && defined(WIN32) && !defined(C_HX_DOS)
 #include "fluidsynth_priv.h"
 #include "fluid_hash.h"
 #include "fluid_list.h"
