@@ -18,10 +18,12 @@
  * 02110-1301, USA
  */
 
-#if !C_FLUIDSYNTH && defined(WIN32)
+#define FLUIDINC
+#include "config.h"
+#if !C_FLUIDSYNTH && defined(WIN32) && !defined(C_HX_DOS)
 #include "fluid_sys.h"
 
-#if (WIN32)
+#if defined(WIN32)
 #include <Winsock2.h>
 #endif
 
@@ -1299,10 +1301,11 @@ static void fluid_server_socket_run (void *data)
     }
     else
     {
-#ifdef IPV6
+#if defined(IPV6) && !defined(__MINGW32__)
       inet_ntop(AF_INET6, &addr.sin6_addr, straddr, sizeof(straddr));
 #else
-      inet_ntop(AF_INET, &addr.sin_addr, straddr, sizeof(straddr));
+      strcpy(straddr, inet_ntoa(addr.sin_addr));
+      //inet_ntop(AF_INET, &addr.sin_addr, straddr, sizeof(straddr));
 #endif
       r = server_socket->func (server_socket->data, client_socket,
                                straddr);
